@@ -41,7 +41,7 @@ int main_cabs(int argc, char* argv[])
 	unmap_cfl(DIMS, dims, idata);
 	unmap_cfl(DIMS, dims, odata);
 
-	exit(0);
+	return 0;
 }
 
 
