@@ -59,7 +59,7 @@ int main_extract(int argc, char* argv[])
 
 	unmap_cfl(DIMS, in_dims, in_data);
 	unmap_cfl(DIMS, out_dims, out_data);
-	exit(0);
+	return 0;
 }
 
 
