@@ -58,7 +58,7 @@ int main_fftmod(int argc, char* argv[])
 
 	unmap_cfl(N, dims, idata);
 	unmap_cfl(N, dims, odata);
-	exit(0);
+	return 0;
 }
 
 
