@@ -1,11 +1,11 @@
 /* Copyright 2013-2018. The Regents of the University of California.
- * Copyright 2016-2017. Martin Uecker.
+ * Copyright 2016-2018. Martin Uecker.
  * Copyright 2017. University of Oxford.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors: 
- * 2012-2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2012-2018 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2014, 2017-2018 Jon Tamir <jtamir@eecs.berkeley.edu>
  * 2017 Sofia Dimoudi <sofia.dimoudi@cardiov.ox.ac.uk>
  */
@@ -39,13 +39,13 @@
 
 void operator_iter(iter_op_data* _data, float* dst, const float* src)
 {
-	struct iter_op_op* data = CAST_DOWN(iter_op_op, _data);
+	auto data = CAST_DOWN(iter_op_op, _data);
 	operator_apply_unchecked(data->op, (complex float*)dst, (const complex float*)src);
 }
 
 void operator_p_iter(iter_op_data* _data, float rho, float* dst, const float* src)
 {
-	struct iter_op_p_op* data = CAST_DOWN(iter_op_p_op, _data);
+	auto data = CAST_DOWN(iter_op_p_op, _data);
 	operator_p_apply_unchecked(data->op, rho, (complex float*)dst, (const complex float*)src);
 }
 
@@ -85,14 +85,13 @@ void iter2_conjgrad(iter_conf* _conf,
 		long size, float* image, const float* image_adj,
 		struct iter_monitor_s* monitor)
 {
-
 	assert(0 == D);
 	assert(NULL == prox_ops);
 	assert(NULL == ops);
 	assert(NULL == biases);
 	UNUSED(xupdate_op);
 
-	struct iter_conjgrad_conf* conf = CAST_DOWN(iter_conjgrad_conf, _conf);
+	auto conf = CAST_DOWN(iter_conjgrad_conf, _conf);
 
 	float eps = md_norm(1, MD_DIMS(size), image_adj);
 
@@ -117,7 +116,6 @@ void iter2_ist(iter_conf* _conf,
 		long size, float* image, const float* image_adj,
 		struct iter_monitor_s* monitor)
 {
-
 	assert(D == 1);
 	assert(NULL != prox_ops[0]);
 	assert(NULL == biases);
@@ -128,7 +126,7 @@ void iter2_ist(iter_conf* _conf,
 #endif
 	UNUSED(xupdate_op);
 
-	struct iter_ist_conf* conf = CAST_DOWN(iter_ist_conf, _conf);
+	auto conf = CAST_DOWN(iter_ist_conf, _conf);
 
 	float eps = md_norm(1, MD_DIMS(size), image_adj);
 
@@ -156,7 +154,6 @@ void iter2_fista(iter_conf* _conf,
 		long size, float* image, const float* image_adj,
 		struct iter_monitor_s* monitor)
 {
-
 	assert(D == 1);
 	assert(NULL == biases);
 #if 0
@@ -166,7 +163,7 @@ void iter2_fista(iter_conf* _conf,
 #endif
 	UNUSED(xupdate_op);
 
-	struct iter_fista_conf* conf = CAST_DOWN(iter_fista_conf, _conf);
+	auto conf = CAST_DOWN(iter_fista_conf, _conf);
 
 	float eps = md_norm(1, MD_DIMS(size), image_adj);
 
@@ -196,7 +193,6 @@ void iter2_chambolle_pock(iter_conf* _conf,
 		long size, float* image, const float* image_adj,
 		struct iter_monitor_s* monitor)
 {
-
 	assert(D == 2);
 	assert(NULL == biases);
 	assert(NULL == normaleq_op);
@@ -204,7 +200,7 @@ void iter2_chambolle_pock(iter_conf* _conf,
 	UNUSED(xupdate_op);
 	UNUSED(image_adj);
 
-	struct iter_chambolle_pock_conf* conf = CAST_DOWN(iter_chambolle_pock_conf, _conf);
+	auto conf = CAST_DOWN(iter_chambolle_pock_conf, _conf);
 
 	const struct iovec_s* iv = linop_domain(ops[1]);
 	const struct iovec_s* ov = linop_codomain(ops[1]);
@@ -242,7 +238,7 @@ void iter2_admm(iter_conf* _conf,
 		long size, float* image, const float* image_adj,
 		struct iter_monitor_s* monitor)
 {
-	struct iter_admm_conf* conf = CAST_DOWN(iter_admm_conf, _conf);
+	auto conf = CAST_DOWN(iter_admm_conf, _conf);
 
 	struct admm_plan_s admm_plan = {
 
@@ -316,7 +312,7 @@ void iter2_pocs(iter_conf* _conf,
 		long size, float* image, const float* image_adj,
 		struct iter_monitor_s* monitor)
 {
-	struct iter_pocs_conf* conf = CAST_DOWN(iter_pocs_conf, _conf);
+	auto conf = CAST_DOWN(iter_pocs_conf, _conf);
 
 	assert(NULL == normaleq_op);
 	assert(NULL == ops);
@@ -350,7 +346,7 @@ void iter2_niht(iter_conf* _conf,
 
 	assert(D == 1);
 	
-	struct iter_niht_conf* conf = CAST_DOWN(iter_niht_conf, _conf);
+	auto conf = CAST_DOWN(iter_niht_conf, _conf);
   
 	struct niht_conf_s niht_conf = {
     
@@ -397,7 +393,7 @@ void iter2_call_iter(iter_conf* _conf,
 
 	UNUSED(xupdate_op);
 
-	struct iter_call_s* it = CAST_DOWN(iter_call_s, _conf);
+	auto it = CAST_DOWN(iter_call_s, _conf);
 
 	it->fun(it->_conf, normaleq_op, (1 == D) ? prox_ops[0] : NULL,
 		size, image, image_adj, monitor);
