@@ -1,10 +1,10 @@
 /* Copyright 2013-2014. The Regents of the University of California.
- * Copyright 2016-2017. Martin Uecker.
+ * Copyright 2016-2018. Martin Uecker.
  * All rights reserved. Use of this source code is governed by 
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2012-2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2012-2018 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  */
 
 #include <assert.h>
@@ -32,6 +32,7 @@ const struct iter3_irgnm_conf iter3_irgnm_defaults = {
 
 	.iter = 8,
 	.alpha = 1.,
+	.alpha_min = 0.,
 	.redu = 2.,
 
 	.cgiter = 100,
@@ -45,12 +46,9 @@ struct irgnm_s {
 
 	INTERFACE(iter_op_data);
 
-	struct iter_op_s frw;
 	struct iter_op_s der;
 	struct iter_op_s adj;
 
-	struct iter_op_s cb;
-
 	float* tmp;
 
 	long size;
@@ -64,7 +62,7 @@ DEF_TYPEID(irgnm_s);
 
 static void normal(iter_op_data* _data, float* dst, const float* src)
 {
-	struct irgnm_s* data = CAST_DOWN(irgnm_s, _data);
+	auto data = CAST_DOWN(irgnm_s, _data);
 
 	iter_op_call(data->der, data->tmp, src);
 	iter_op_call(data->adj, dst, data->tmp);
@@ -72,7 +70,7 @@ static void normal(iter_op_data* _data, float* dst, const float* src)
 
 static void inverse(iter_op_data* _data, float alpha, float* dst, const float* src)
 {
-	struct irgnm_s* data = CAST_DOWN(irgnm_s, _data);
+	auto data = CAST_DOWN(irgnm_s, _data);
 
 	md_clear(1, MD_DIMS(data->size), dst, FL_SIZE);
 
@@ -89,28 +87,6 @@ static void inverse(iter_op_data* _data, float alpha, float* dst, const float* s
 			(struct iter_op_s){ normal, CAST_UP(data) }, dst, src, NULL);
 }
 
-static void forward(iter_op_data* _data, float* dst, const float* src)
-{
-	struct irgnm_s* data = CAST_DOWN(irgnm_s, _data);
-
-	iter_op_call(data->frw, dst, src);
-}
-
-static void adjoint(iter_op_data* _data, float* dst, const float* src)
-{
-	struct irgnm_s* data = CAST_DOWN(irgnm_s, _data);
-
-	iter_op_call(data->adj, dst, src);
-}
-
-static void callback(iter_op_data* _data, float* dst, const float* src)
-{
-	struct irgnm_s* data = CAST_DOWN(irgnm_s, _data);
-
-	iter_op_call(data->cb, dst, src);
-}
-
-
 void iter3_irgnm(iter3_conf* _conf,
 		struct iter_op_s frw,
 		struct iter_op_s der,
@@ -119,21 +95,21 @@ void iter3_irgnm(iter3_conf* _conf,
 		long M, const float* src,
 		 struct iter_op_s cb)
 {
-	struct iter3_irgnm_conf* conf = CAST_DOWN(iter3_irgnm_conf, _conf);
+	auto conf = CAST_DOWN(iter3_irgnm_conf, _conf);
 
 	float* tmp = md_alloc_sameplace(1, MD_DIMS(M), FL_SIZE, src);
-	struct irgnm_s data = { { &TYPEID(irgnm_s) }, frw, der, adj, cb, tmp, N, conf->cgiter, conf->cgtol, conf->nlinv_legacy };
 
-	irgnm(conf->iter, conf->alpha, conf->redu, N, M, select_vecops(src),
-		(struct iter_op_s){ forward, CAST_UP(&data) },
-		(struct iter_op_s){ adjoint, CAST_UP(&data) },
+	struct irgnm_s data = { { &TYPEID(irgnm_s) }, der, adj, tmp, N, conf->cgiter, conf->cgtol, conf->nlinv_legacy };
+
+	irgnm(conf->iter, conf->alpha, conf->alpha_min, conf->redu, N, M, select_vecops(src),
+		frw,
+		adj,
 		(struct iter_op_p_s){ inverse, CAST_UP(&data) },
 		dst, ref, src,
-		(struct iter_op_s){ callback, CAST_UP(&data) });
-
+		cb);
 	md_free(tmp);
-}
 
+}
 
 
 void iter3_landweber(iter3_conf* _conf,
@@ -143,7 +119,7 @@ void iter3_landweber(iter3_conf* _conf,
 		long N, float* dst, const float* ref,
 		long M, const float* src)
 {
-	struct iter3_landweber_conf* conf = CAST_DOWN(iter3_landweber_conf, _conf);
+	auto conf = CAST_DOWN(iter3_landweber_conf, _conf);
 
 	assert(NULL == der.fun);
 	assert(NULL == ref);
