@@ -7,6 +7,7 @@
 
 struct iter3_conf_s;
 struct iter_op_s;
+struct iter_nlop_s;
 struct nlop_s;
 
 typedef void iter4_fun_f(iter3_conf* _conf,
@@ -20,7 +21,14 @@ typedef void iter4_irgnm_f(iter3_conf* _conf,
 			   long M, const float* src,
 			   struct iter_op_s cb);
 
+typedef void iter4_altmin_f(iter3_conf* _conf,
+			struct nlop_s* nlop,
+			long NI, float* dst[NI],
+			long M, const float* src,
+			struct iter_nlop_s cb);
+
 iter4_irgnm_f iter4_irgnm;
+iter4_altmin_f iter4_altmin;
 iter4_fun_f iter4_landweber;
 
 
