@@ -1,10 +1,10 @@
 /* Copyright 2014. The Regents of the University of California.
- * Copyright 2016. Martin Uecker.
+ * Copyright 2016-2018. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2014-2016 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2014-2018 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2014 Frank Ong <frankong@berkeley.edu>
  */
 
@@ -14,9 +14,11 @@
 
 #include "num/multind.h"
 #include "num/flpmath.h"
+#include "num/iovec.h"
 #include "num/ops.h"
 
 #include "misc/misc.h"
+#include "misc/shrdptr.h"
 #include "misc/debug.h"
 #include "misc/shrdptr.h"
 
@@ -48,7 +50,7 @@ static DEF_TYPEID(shared_data_s);
 
 static void shared_del(const operator_data_t* _data)
 {
-	struct shared_data_s* data = CAST_DOWN(shared_data_s, _data);
+	auto data = CAST_DOWN(shared_data_s, _data);
 
 	shared_ptr_destroy(&data->sptr);
 	
@@ -57,7 +59,7 @@ static void shared_del(const operator_data_t* _data)
 
 static void shared_apply(const operator_data_t* _data, unsigned int N, void* args[N])
 {
-	struct shared_data_s* data = CAST_DOWN(shared_data_s, _data);
+	auto data = CAST_DOWN(shared_data_s, _data);
 
 	assert(2 == N);
 	debug_trace("ENTER %p\n", data->u.apply);
@@ -67,7 +69,7 @@ static void shared_apply(const operator_data_t* _data, unsigned int N, void* arg
 
 static void shared_apply_p(const operator_data_t* _data, float lambda, complex float* dst, const complex float* src)
 {
-	struct shared_data_s* data = CAST_DOWN(shared_data_s, _data);
+	auto data = CAST_DOWN(shared_data_s, _data);
 
 	debug_trace("ENTER %p\n", data->u.apply_p);
 	data->u.apply_p(data->data, lambda, dst, src);
@@ -77,7 +79,7 @@ static void shared_apply_p(const operator_data_t* _data, float lambda, complex f
 
 static void sptr_del(const struct shared_ptr_s* p)
 {
-	struct shared_data_s* data = CONTAINER_OF(p, struct shared_data_s, sptr);
+	auto data = CONTAINER_OF(p, struct shared_data_s, sptr);
 
 	data->del(data->data);
 }
@@ -184,7 +186,8 @@ struct linop_s* linop_create(unsigned int ON, const long odims[ON], unsigned int
  */
 const linop_data_t* linop_get_data(const struct linop_s* ptr)
 {
-	return ((struct shared_data_s*)operator_get_data(ptr->forward))->data;
+	auto sdata = CAST_MAYBE(shared_data_s, operator_get_data(ptr->forward));
+	return sdata == NULL ? NULL : sdata->data;
 }
 
 
@@ -370,6 +373,40 @@ const struct iovec_s* linop_codomain(const struct linop_s* op)
 }
 
 
+
+
+
+struct linop_s* linop_null_create2(unsigned int N, const long odims[N], const long ostrs[N], const long idims[N], const long istrs[N])
+{
+	PTR_ALLOC(struct linop_s, c);
+
+	const struct operator_s* nudo = operator_null_create2(N, idims, istrs);
+	const struct operator_s* zedo = operator_zero_create2(N, idims, istrs);
+	const struct operator_s* nuco = operator_null_create2(N, odims, ostrs);
+	const struct operator_s* zeco = operator_zero_create2(N, odims, ostrs);
+
+	c->forward = operator_combi_create(2, MAKE_ARRAY(zeco, nudo));
+	c->adjoint = operator_combi_create(2, MAKE_ARRAY(zedo, nuco));
+	c->normal = operator_combi_create(2, MAKE_ARRAY(zedo, nudo));
+	c->norm_inv = NULL;
+
+	operator_free(nudo);
+	operator_free(zedo);
+	operator_free(nuco);
+	operator_free(zeco);
+
+	return PTR_PASS(c);
+}
+
+
+
+struct linop_s* linop_null_create(unsigned int N, const long odims[N], const long idims[N])
+{
+	return linop_null_create2(N, odims, MD_STRIDES(N, odims, CFL_SIZE),
+					idims, MD_STRIDES(N, idims, CFL_SIZE));
+}
+
+
 /**
  * Create chain of linear operators.
  * C = B A 
@@ -400,6 +437,17 @@ struct linop_s* linop_chain(const struct linop_s* a, const struct linop_s* b)
 }
 
 
+struct linop_s* linop_chain_FF(const struct linop_s* a, const struct linop_s* b)
+{
+	struct linop_s* x = linop_chain(a, b);
+
+	linop_free(a);
+	linop_free(b);
+
+	return x;
+}
+
+
 struct linop_s* linop_chainN(unsigned int N, struct linop_s* a[N])
 {
 	assert(N > 0);
@@ -407,7 +455,7 @@ struct linop_s* linop_chainN(unsigned int N, struct linop_s* a[N])
 	if (1 == N)
 		return a[0];
 
-	return linop_chain(a[0], linop_chainN(N - 1, a + 1));
+	return linop_chain(a[0], linop_chainN(N - 1, a + 1));	// FIXME: free intermed.
 }
 
 
@@ -431,6 +479,8 @@ struct linop_s* linop_loop(unsigned int D, const long dims[D], struct linop_s* o
  */
 void linop_free(const struct linop_s* op)
 {
+	if (NULL == op)
+		return;
 	operator_free(op->forward);
 	operator_free(op->adjoint);
 	operator_free(op->normal);
@@ -439,3 +489,84 @@ void linop_free(const struct linop_s* op)
 }
 
 
+
+
+struct plus_data_s {
+
+	INTERFACE(linop_data_t);
+
+	const struct linop_s* a;
+	const struct linop_s* b;
+};
+
+static DEF_TYPEID(plus_data_s);
+
+static void plus_apply(const linop_data_t* _data, complex float* dst, const complex float* src)
+{
+	auto data = CAST_DOWN(plus_data_s, _data);
+	auto iov = linop_codomain(data->a);
+
+	complex float* tmp = md_alloc_sameplace(iov->N, iov->dims, iov->size, dst);
+
+	linop_forward_unchecked(data->a, dst, src);
+	linop_forward_unchecked(data->b, tmp, src);
+
+	md_zadd(iov->N, iov->dims, dst, dst, tmp);
+	md_free(tmp);
+}
+
+static void plus_adjoint(const linop_data_t* _data, complex float* dst, const complex float* src)
+{
+	auto data = CAST_DOWN(plus_data_s, _data);
+	auto iov = linop_domain(data->a);
+
+	complex float* tmp = md_alloc_sameplace(iov->N, iov->dims, iov->size, dst);
+
+	linop_adjoint_unchecked(data->a, dst, src);
+	linop_adjoint_unchecked(data->b, tmp, src);
+
+	md_zadd(iov->N, iov->dims, dst, dst, tmp);
+	md_free(tmp);
+}
+
+
+static void plus_free(const linop_data_t* _data)
+{
+	auto data = CAST_DOWN(plus_data_s, _data);
+
+	linop_free(data->a);
+	linop_free(data->b);
+
+	xfree(data);
+}
+
+struct linop_s* linop_plus(const struct linop_s* a, const struct linop_s* b)
+{
+#if 1
+	// detect null operations and just clone
+
+	if (operator_zero_or_null_p(a->forward))
+		return (struct linop_s*)linop_clone(b);
+
+	if (operator_zero_or_null_p(b->forward))
+		return (struct linop_s*)linop_clone(a);
+#endif
+
+	auto bdo = linop_domain(b);
+	assert(CFL_SIZE == bdo->size);
+	iovec_check(linop_domain(a), bdo->N, bdo->dims, bdo->strs);
+
+	auto bco = linop_codomain(b);
+	assert(CFL_SIZE == bco->size);
+	iovec_check(linop_codomain(a), bco->N, bco->dims, bco->strs);
+
+	PTR_ALLOC(struct plus_data_s, data);
+	SET_TYPEID(plus_data_s, data);
+
+	data->a = linop_clone(a);
+	data->b = linop_clone(b);
+
+	return linop_create(bco->N, bco->dims, bdo->N, bdo->dims, CAST_UP(PTR_PASS(data)), plus_apply, plus_adjoint, NULL, NULL, plus_free);
+}
+
+
