@@ -1,5 +1,5 @@
 /* Copyright 2014-2015. The Regents of the University of California.
- * Copyright 2016. Martin Uecker.
+ * Copyright 2016-2018. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  */
@@ -68,6 +68,8 @@ extern void linop_norm_inv_unchecked(const struct linop_s* op, float lambda, com
 extern struct linop_s* linop_chain(const struct linop_s* a, const struct linop_s* b);
 extern struct linop_s* linop_chainN(unsigned int N, struct linop_s* x[N]);
 
+extern struct linop_s* linop_chain_FF(const struct linop_s* a, const struct linop_s* b);
+
 struct iovec_s;
 extern const struct iovec_s* linop_domain(const struct linop_s* x);
 extern const struct iovec_s* linop_codomain(const struct linop_s* x);
@@ -78,10 +80,11 @@ extern const struct linop_s* linop_clone(const struct linop_s* x);
 extern struct linop_s* linop_loop(unsigned int D, const long dims[D], struct linop_s* op);
 
 
-// extern const struct linop_s* linop_identity(unsigned int N, const long dims[N]);
-// extern const struct linop_s* linop_matrix(unsigned int N, const long out_dims[N], const long in_dims[N], const long matrix_dims[N], const _Complex float* matrix, _Bool use_gpu);
-extern const struct linop_s* linop_join(unsigned int D, const struct linop_s* a, const struct linop_s* b);
 
+extern struct linop_s* linop_null_create2(unsigned int N, const long odims[N], const long ostrs[N], const long idims[N], const long istrs[N]);
+extern struct linop_s* linop_null_create(unsigned int N, const long odims[N], const long idims[N]);
+
+extern struct linop_s* linop_plus(const struct linop_s* a, const struct linop_s* b);
 
 
 #include "misc/cppwrap.h"
