@@ -184,7 +184,7 @@ int main_lrmatrix(int argc, char* argv[])
         }
 	
 	const struct operator_p_s* sum_prox = prox_lineq_create( sum_op, idata );
-	const struct operator_p_s* lr_prox = lrthresh_create(odims, randshift, mflags, (const long (*)[])blkdims, 1., noise, remove_mean);
+	const struct operator_p_s* lr_prox = lrthresh_create(odims, randshift, mflags, (const long (*)[])blkdims, 1., noise, remove_mean, false);
 
         assert(use_gpu == false);
 
@@ -246,6 +246,6 @@ int main_lrmatrix(int argc, char* argv[])
 
 	double end_time = timestamp();
 	debug_printf(DP_INFO, "Total Time: %f\n", end_time - start_time);
-	exit(0);
+	return 0;
 }
 
