@@ -55,24 +55,24 @@ int main_nlinv(int argc, char* argv[])
 
 		OPT_UINT('i', &conf.iter, "iter", "Number of Newton steps"),
 		OPT_FLOAT('R', &conf.redu, "", "(reduction factor)"),
+		OPT_FLOAT('M', &conf.alpha_min, "", "(minimum for regularization)"),
 		OPT_INT('d', &debug_level, "level", "Debug level"),
 		OPT_SET('c', &conf.rvc, "Real-value constraint"),
 		OPT_CLEAR('N', &normalize, "Do not normalize image with coil sensitivities"),
-		OPT_UINT('m', &nmaps, "nmaps", "Number of ENLIVE maps"),
+		OPT_UINT('m', &nmaps, "nmaps", "Number of ENLIVE maps to use in reconsctruction"),
 		OPT_CLEAR('U', &combine, "Do not combine ENLIVE maps in output"),
 		OPT_FLOAT('f', &restrict_fov, "FOV", ""),
 		OPT_STRING('p', &psf, "PSF", ""),
 		OPT_STRING('I', &init_file, "file", "File for initialization"),
 		OPT_SET('g', &conf.usegpu, "use gpu"),
 		OPT_SET('S', &scale_im, "Re-scale image after reconstruction"),
-		OPT_SET('P', &conf.pattern_for_each_coil, "(supplied psf is different for each coil)"),
 		OPT_FLOAT('a', &conf.a, "", "(a in 1 + a * \\Laplace^-b/2)"),
 		OPT_FLOAT('b', &conf.b, "", "(b in 1 + a * \\Laplace^-b/2)"),
+		OPT_SET('P', &conf.pattern_for_each_coil, "(supplied psf is different for each coil)"),
 	};
 
 	cmdline(&argc, argv, 2, 3, usage_str, help_str, ARRAY_SIZE(opts), opts);
 
-
 	if (4 == argc)
 		out_sens = true;
 
@@ -83,11 +83,13 @@ int main_nlinv(int argc, char* argv[])
 	long ksp_dims[DIMS];
 	complex float* kspace_data = load_cfl(argv[1], DIMS, ksp_dims);
 
-	// SMS
+	// FIXME: SMS should not be the default
+
 	if (1 != ksp_dims[SLICE_DIM]) {
 
 		debug_printf(DP_INFO, "SMS-NLINV reconstruction. Multiband factor: %d\n", ksp_dims[SLICE_DIM]);
 		fftmod(DIMS, ksp_dims, SLICE_FLAG, kspace_data, kspace_data); // fftmod to get correct slice order in output
+		conf.sms = true;
 	}
 
 	// The only multimap we understand with is the one we do ourselves, where
@@ -106,25 +108,21 @@ int main_nlinv(int argc, char* argv[])
 
 
 	long img_dims[DIMS];
-	md_select_dims(DIMS, FFT_FLAGS|MAPS_FLAG|SLICE_FLAG, img_dims, sens_dims );
+	md_select_dims(DIMS, FFT_FLAGS|MAPS_FLAG|SLICE_FLAG, img_dims, sens_dims);
 
 	long img_strs[DIMS];
 	md_calc_strides(DIMS, img_strs, img_dims, CFL_SIZE);
 
 	long img_output_dims[DIMS];
 	md_select_dims(DIMS, FFT_FLAGS|SLICE_FLAG, img_output_dims, sens_dims);
-
 	if (!combine)
 		img_output_dims[MAPS_DIM] = nmaps;
 
 	long img_output_strs[DIMS];
 	md_calc_strides(DIMS, img_output_strs, img_output_dims, CFL_SIZE);
 
-
 	complex float* img_output = create_cfl(argv[2], DIMS, img_output_dims);
-
 	md_clear(DIMS, img_output_dims, img_output, CFL_SIZE);
-
 	complex float* img = md_alloc(DIMS, img_dims, CFL_SIZE);
 
 	long msk_dims[DIMS];
@@ -134,11 +132,9 @@ int main_nlinv(int argc, char* argv[])
 	md_calc_strides(DIMS, msk_strs, msk_dims, CFL_SIZE);
 
 	complex float* mask = NULL;
-	complex float* norm = md_alloc(DIMS, img_dims, CFL_SIZE);
 
 	complex float* sens = (out_sens ? create_cfl : anon_cfl)(out_sens ? argv[3] : "", DIMS, sens_dims);
 
-
 	// initialization
 	if (NULL != init_file) {
 
@@ -164,15 +160,16 @@ int main_nlinv(int argc, char* argv[])
 
 	if (NULL != psf) {
 
-		pattern = load_cfl(psf, DIMS, pat_dims);
+		complex float* tmp_psf =load_cfl(psf, DIMS, pat_dims);
+		pattern = anon_cfl("", DIMS, pat_dims);
+
+		md_copy(DIMS, pat_dims, pattern, tmp_psf, CFL_SIZE);
+		unmap_cfl(DIMS, pat_dims, tmp_psf);
 		// FIXME: check compatibility
 
 		if (conf.pattern_for_each_coil) {
-
-			assert(1 != pat_dims[COIL_DIM]);
-
+			assert( 1 != pat_dims[COIL_DIM] );
 		} else {
-
 			if (-1 == restrict_fov)
 				restrict_fov = 0.5;
 
@@ -212,17 +209,19 @@ int main_nlinv(int argc, char* argv[])
 		mask = compute_mask(DIMS, msk_dims, restrict_dims);
 	}
 
+	complex float* ref = NULL;
+
 #ifdef  USE_CUDA
 	if (conf.usegpu) {
 
 		complex float* kspace_gpu = md_alloc_gpu(DIMS, ksp_dims, CFL_SIZE);
 		md_copy(DIMS, ksp_dims, kspace_gpu, kspace_data, CFL_SIZE);
 
-		noir_recon(&conf, sens_dims, img, sens, pattern, mask, kspace_gpu);
+		noir_recon(&conf, sens_dims, img, sens, ref, pattern, mask, kspace_gpu);
 		md_free(kspace_gpu);
 	} else
 #endif
-	noir_recon(&conf, sens_dims, img, sens, pattern, mask, kspace_data);
+		noir_recon(&conf, sens_dims, img, sens, ref, pattern, mask, kspace_data);
 
 
 	// image output
@@ -235,7 +234,6 @@ int main_nlinv(int argc, char* argv[])
 
 			md_zfmac2(DIMS, sens_dims, ksp_strs, buf, img_strs, img, sens_strs, sens);
 			md_zrss(DIMS, ksp_dims, COIL_FLAG, img_output, buf);
-
 		} else {
 
 			md_zfmac2(DIMS, sens_dims, sens_strs, buf, img_strs, img, sens_strs, sens);
@@ -243,7 +241,7 @@ int main_nlinv(int argc, char* argv[])
 		}
 		md_zmul2(DIMS, img_output_dims, img_output_strs, img_output, img_output_strs, img_output, msk_strs, mask);
 
-		if ((1 == nmaps) || !combine) {
+		if (1 == nmaps || !combine) {
 
 			//restore phase
 			md_zphsr(DIMS, img_output_dims, buf, img);
@@ -251,7 +249,6 @@ int main_nlinv(int argc, char* argv[])
 		}
 
 		md_free(buf);
-
 	} else {
 
 		if (combine) {
@@ -268,8 +265,6 @@ int main_nlinv(int argc, char* argv[])
 	if (scale_im)
 		md_zsmul(DIMS, img_output_dims, img_output, img_output, 1. / scaling);
 
-
-	md_free(norm);
 	md_free(mask);
 	md_free(img);
 
@@ -279,9 +274,7 @@ int main_nlinv(int argc, char* argv[])
 	unmap_cfl(DIMS, ksp_dims, kspace_data);
 
 	double recosecs = timestamp() - start_time;
-
 	debug_printf(DP_DEBUG2, "Total Time: %.2f s\n", recosecs);
-
 	exit(0);
 }
 
