@@ -27,8 +27,8 @@ extern void grid2(const struct grid_conf_s* conf, unsigned int D, const long trj
 extern void grid2H(const struct grid_conf_s* conf, unsigned int D, const long trj_dims[__VLA(D)], const complex float* traj, const long ksp_dims[__VLA(D)], complex float* dst, const long grid_dims[__VLA(D)], const complex float* grid);
 
 
-extern void grid_pointH(unsigned int ch, const long dims[3], const float pos[3], complex float val[__VLA(ch)], const complex float* src, _Bool periodic, float width, int kb_size, const float kb_table[__VLA(kb_size + 1)]);
-extern void grid_point(unsigned int ch, const long dims[3], const float pos[3], complex float* dst, const complex float val[__VLA(ch)], _Bool periodic, float width, int kb_size, const float kb_table[__VLA(kb_size + 1)]);
+extern void grid_pointH(unsigned int ch, int N, const long dims[__VLA(N)], const float pos[__VLA(N)], complex float val[__VLA(ch)], const complex float* src, _Bool periodic, float width, int kb_size, const float kb_table[__VLA(kb_size + 1)]);
+extern void grid_point(unsigned int ch, int N, const long dims[__VLA(N)], const float pos[__VLA(N)], complex float* dst, const complex float val[__VLA(ch)], _Bool periodic, float width, int kb_size, const float kb_table[__VLA(kb_size + 1)]);
 
 extern double calc_beta(float os, float width);
 
