@@ -1,4 +1,5 @@
 /* Copyright 2014-2015. The Regents of the University of California.
+ * Copyright 2016-2018. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  */
@@ -13,6 +14,9 @@ struct nufft_conf_s {
 	_Bool toeplitz; ///< Toeplitz embedding boolean for A^T A
 	_Bool pcycle; /// < Phase cycling
 	_Bool periodic;
+	_Bool lowmem;
+	unsigned long flags;
+	unsigned long cfft;
 };
 
 extern struct nufft_conf_s nufft_conf_defaults;
@@ -26,20 +30,30 @@ extern struct linop_s* nufft_create(unsigned int N,			///< Number of dimensions
 				    const _Complex float* weights,	///< Weights, ex, density-compensation
 				    struct nufft_conf_s conf);		///< NUFFT configuration
 
-extern void estimate_im_dims(unsigned int N,			///< Number of dimensions
-			     long dims[3],			///< Output estimated image dimensions
-			     const long tdims[__VLA(N)],	///< Trajectory dimesion
-			     const _Complex float* traj);	///< Trajectory
-
+extern struct linop_s* nufft_create2(unsigned int N,
+			     const long ksp_dims[N],
+			     const long cim_dims[N],
+			     const long traj_dims[N],
+			     const complex float* traj,
+			     const long wgh_dims[N],
+			     const complex float* weights,
+			     const long bas_dims[N],
+			     const complex float* basis,
+			     struct nufft_conf_s conf);
 
 extern _Complex float* compute_psf(unsigned int N,
 				   const long img2_dims[__VLA(N)],
 				   const long trj_dims[__VLA(N)],
 				   const complex float* traj,
-				   const complex float* weights);
+				   const long bas_dims[__VLA(N)],
+				   const complex float* basis,
+				   const long wgh_dims[__VLA(N)],
+				   const complex float* weights,
+				   _Bool periodic);
 
+extern void estimate_im_dims(int N, unsigned long flags, long dims[__VLA(N)], const long tdims[__VLA(N)], const complex float* traj);
 
-extern const struct operator_s* nufft_precond_create( const struct linop_s* nufft_op );
+extern const struct operator_s* nufft_precond_create(const struct linop_s* nufft_op);
 
 #include "misc/cppwrap.h"
 
