@@ -1,4 +1,4 @@
-/* Copyright 2013-2015 The Regents of the University of California.
+/* Copyright 2013-2018 The Regents of the University of California.
  * Copyright 2016-2017. Martin Uecker.
  * Copyright 2017. University of Oxford.
  * All rights reserved. Use of this source code is governed by
@@ -8,7 +8,7 @@
  * 2012-2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2013 Dara Bahri <dbahri123@gmail.com>
  * 2014 Frank Ong <frankong@berkeley.edu>
- * 2014-2015 Jonathan Tamir <jtamir@eecs.berkeley.edu>
+ * 2014-2018 Jonathan Tamir <jtamir@eecs.berkeley.edu>
  * 2016 Siddharth Iyer <sid8795@gmail.com>
  * 2017 Sofia Dimoudi <sofia.dimoudi@cardiov.ox.ac.uk>
  *
@@ -1017,12 +1017,12 @@ void md_zfloat2double(unsigned int D, const long dims[D], complex double* dst, c
  */
 void md_tenmul_dims(unsigned int D, long max_dims[D], const long out_dims[D], const long in1_dims[D], const long in2_dims[D])
 {
-	md_max_dims(D, ~0u, max_dims, in1_dims, out_dims);
+	md_max_dims(D, ~0lu, max_dims, in1_dims, out_dims);
 
 	long max2_dims[D];
-	md_max_dims(D, ~0u, max2_dims, in2_dims, out_dims);
+	md_max_dims(D, ~0lu, max2_dims, in2_dims, out_dims);
 
-	assert(md_check_compat(D, 0u, max_dims, max2_dims));
+	assert(md_check_compat(D, 0lu, max_dims, max2_dims));
 }
 
 
@@ -1432,6 +1432,32 @@ void md_min2(unsigned int D, const long dims[D], const long ostr[D], float* optr
 }
 
 
+/**
+ * Max of inputs (without strides)
+ *
+ * optr = max(iptr1, iptr2)
+ */
+void md_zmax(unsigned int D, const long dims[D], complex float* optr, const complex float* iptr1, const complex float* iptr2)
+{
+	long strs[D];
+	md_calc_strides(D, strs, dims, CFL_SIZE);
+
+	md_zmax2(D, dims, strs, optr, strs, iptr1, strs, iptr2);
+}
+
+
+/**
+ * Max of inputs (with strides)
+ *
+ * optr = max(iptr1, iptr2)
+ */
+void md_zmax2(unsigned int D, const long dims[D], const long ostr[D], complex float* optr, const long istr1[D], const complex float* iptr1, const long istr2[D], const complex float* iptr2)
+{
+	MAKE_Z3OP(zmax, D, dims, ostr, optr, istr1, iptr1, istr2, iptr2);
+}
+
+
+
 
 /**
  * Multiply complex array with a scalar and add to output (without strides)
@@ -1757,6 +1783,29 @@ void md_zcmp(unsigned int D, const long dims[D], complex float* optr, const comp
 	make_z3op_simple(md_zcmp2, D, dims, optr, iptr1, iptr2);
 }
 
+/**
+ * Elementwise less than or equal to (with strides)
+ *
+ * optr = (iptr1 <= iptr2)
+ */
+void md_zlessequal2(unsigned int D, const long dims[D], const long ostr[D], complex float* optr, const long istr1[D], const complex float* iptr1, const long istr2[D], const complex float* iptr2)
+{
+	MAKE_Z3OP(zle, D, dims, ostr, optr, istr1, iptr1, istr2, iptr2);
+}
+
+
+
+/**
+ * Elementwise less than or equal to (without strides)
+ *
+ * optr = (iptr1 <= iptr2)
+ */
+void md_zlessequal(unsigned int D, const long dims[D], complex float* optr, const complex float* iptr1, const complex float* iptr2)
+{
+	make_z3op_simple(md_zlessequal2, D, dims, optr, iptr1, iptr2);
+}
+
+
 
 
 /**
@@ -1809,6 +1858,29 @@ void md_slessequal(unsigned int D, const long dims[D], float* optr, const float*
 }
 
 
+/**
+ * Elementwise greater than or equal to (with strides)
+ *
+ * optr = (iptr1 => iptr2)
+ */
+void md_zgreatequal2(unsigned int D, const long dims[D], const long ostr[D], complex float* optr, const long istr1[D], const complex float* iptr1, const long istr2[D], const complex float* iptr2)
+{
+	md_zlessequal2(D, dims, ostr, optr, istr2, iptr2, istr1, iptr1);
+}
+
+
+
+/**
+ * Elementwise greater than or equal to (without strides)
+ *
+ * optr = (iptr1 >= iptr2)
+ */
+void md_zgreatequal(unsigned int D, const long dims[D], complex float* optr, const complex float* iptr1, const complex float* iptr2)
+{
+	make_z3op_simple(md_zgreatequal2, D, dims, optr, iptr1, iptr2);
+}
+
+
 
 /**
  * Elementwise greater than or equal to (with strides)
@@ -1861,6 +1933,33 @@ void md_sgreatequal(unsigned int D, const long dims[D], float* optr, const float
 
 
 
+/**
+ * Elementwise greater than or equal to scalar (with strides)
+ *
+ * optr = (iptr >= val)
+ */
+void md_zsgreatequal2(unsigned int D, const long dims[D], const long ostr[D], complex float* optr, const long istr[D], const complex float* iptr, float val)
+{
+	make_z3op_scalar(md_zgreatequal2, D, dims, ostr, optr, istr, iptr, val);
+}
+
+
+
+/**
+ * Elementwise greater than or equal to scalar (without strides)
+ *
+ * optr = (iptr >= val)
+ */
+void md_zsgreatequal(unsigned int D, const long dims[D], complex float* optr, const complex float* iptr, float val)
+{
+	long strs[D];
+	md_calc_strides(D, strs, dims, CFL_SIZE);
+
+	md_zsgreatequal2(D, dims, strs, optr, strs, iptr, val);
+}
+
+
+
 /**
  * Extract unit-norm complex exponentials from complex arrays (with strides)
  *
@@ -1908,6 +2007,32 @@ void md_zexpj(unsigned int D, const long dims[D], complex float* optr, const com
 
 
 
+
+/**
+ * Complex exponential
+ *
+ * optr = zexp(iptr)
+ */
+void md_zexp2(unsigned int D, const long dims[D], const long ostr[D], complex float* optr, const long istr[D], const complex float* iptr)
+{
+	MAKE_Z2OP(zexp, D, dims, ostr, optr, istr, iptr);
+}
+
+
+
+/**
+ * Complex exponential
+ *
+ * optr = zexp(iptr)
+ */
+void md_zexp(unsigned int D, const long dims[D], complex float* optr, const complex float* iptr)
+{
+	make_z2op_simple(md_zexp2, D, dims, optr, iptr);
+}
+
+
+
+
 /**
  * Get argument of complex arrays (with strides)
  *
@@ -1973,6 +2098,7 @@ float md_scalar2(unsigned int D, const long dim[D], const long str1[D], const fl
 
 #ifdef USE_CUDA
 	if (cuda_ondevice(ptr1)) {
+
 		md_copy(1, (long[1]){ 1 }, &ret, retp, DL_SIZE);
 		md_free(retp);
 	}
@@ -2123,6 +2249,7 @@ complex float md_zscalar2(unsigned int D, const long dim[D], const long str1[D],
 
 #ifdef USE_CUDA
 	if (cuda_ondevice(ptr1)) {
+
 		md_copy(1, (long[1]){ 1 }, &ret, retp, CDL_SIZE);
 		md_free(retp);
 	}
@@ -2608,7 +2735,6 @@ void md_zavg2(unsigned int D, const long dims[D], unsigned int flags, const long
 	md_singleton_strides(D, ss);
 	md_zfmac2(D, dims, ostr, optr, istr, iptr, ss, o);
 	md_free(o);
-
 #else
 	md_zaxpy2(D, dims, ostr, optr, 1., istr, iptr);
 #endif
@@ -3147,6 +3273,8 @@ void md_smin(unsigned int D, const long dim[D], float* optr, const float* iptr,
  */
 void md_smax2(unsigned int D, const long dim[D], const long ostr[D], float* optr, const long istr[D], const float* iptr, float val)
 {
+#if 0
+	// slow on GPU due to make_3op_scalar
 #if 0
 	float* tmp = md_alloc_sameplace(D, dim, FL_SIZE, iptr);
 	md_sgreatequal2(D, dim, ostr, tmp, istr, iptr, val);
@@ -3155,6 +3283,35 @@ void md_smax2(unsigned int D, const long dim[D], const long ostr[D], float* optr
 #else
 	make_3op_scalar(md_max2, D, dim, ostr, optr, istr, iptr, val);
 #endif
+#else
+	(void)0;
+
+	NESTED(void, nary_smax, (struct nary_opt_data_s* data, void* ptr[]))
+	{
+		data->ops->smax(data->size, ptr[0], ptr[1], val);
+	};
+
+	optimized_twoop_oi(D, dim, ostr, optr, istr, iptr,
+		(size_t[2]){ FL_SIZE, FL_SIZE }, nary_smax);
+#endif
+}
+
+
+/**
+ * Elementwise maximum of input and scalar (with strides)
+ *
+ * optr = max(val, iptr)
+ */
+void md_zsmax2(unsigned int D, const long dim[D], const long ostr[D], complex float* optr, const long istr[D], const complex float* iptr, float val)
+{
+#if 0
+	complex float* tmp = md_alloc_sameplace(D, dim, CFL_SIZE, iptr);
+	md_zsgreatequal2(D, dim, ostr, tmp, istr, iptr, val);
+	md_zmul2(D, dim, ostr, optr, istr, iptr, istr, tmp);
+	md_free(tmp);
+#else
+	make_z3op_scalar(md_zmax2, D, dim, ostr, optr, istr, iptr, val);
+#endif
 }
 
 
@@ -3323,3 +3480,31 @@ void md_zfftmod(unsigned int D, const long dims[D], complex float* optr, const c
 
 
 
+/**
+ * Sum along selected dimensions
+ *
+ * @param dims -- full dimensions of src image
+ * @param flags -- bitmask for applying the sum, i.e. the dimensions that will not stay
+ */
+void md_zsum(unsigned int D, const long dims[D], unsigned int flags, complex float* dst, const complex float* src)
+{
+	long str1[D];
+	long str2[D];
+	long dims2[D];
+
+	md_select_dims(D, ~flags, dims2, dims);
+
+	md_calc_strides(D, str1, dims, CFL_SIZE);
+	md_calc_strides(D, str2, dims2, CFL_SIZE);
+
+	complex float* ones = md_alloc_sameplace(D, dims, CFL_SIZE, dst);
+	md_zfill(D, dims, ones, 1.);
+
+	md_clear(D, dims2, dst, CFL_SIZE);
+	md_zfmac2(D, dims, str2, dst, str1, src, str1, ones);
+
+	md_free(ones);
+}
+
+
+
