@@ -1,13 +1,13 @@
-/* Copyright 2013-2017. The Regents of the University of California.
+/* Copyright 2013-2018. The Regents of the University of California.
  * Copyright 2014. Joseph Y Cheng.
- * Copyright 2016. Martin Uecker.
+ * Copyright 2016-2017. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors: 
- * 2012-2016	Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2012-2017	Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2014 	Joseph Y Cheng <jycheng@stanford.edu>
- * 2015,2017		Jon Tamir <jtamir@eecs.berkeley.edu>
+ * 2015-2018	Jon Tamir <jtamir@eecs.berkeley.edu>
  *
  * 
  * CUDA support functions. The file exports gpu_ops of type struct vec_ops
@@ -34,6 +34,7 @@
 
 #include "gpuops.h"
 
+#define MiBYTE (1024*1024)
 
 
 static void cuda_error(int line, cudaError_t code)
@@ -42,8 +43,27 @@ static void cuda_error(int line, cudaError_t code)
 	error("cuda error: %d %s \n", line, err_str);
 }
 
+
 #define CUDA_ERROR(x)	({ cudaError_t errval = (x); if (cudaSuccess != errval) cuda_error(__LINE__, errval); })
 
+// Print free and used memory on GPU.
+void print_cuda_meminfo(void)
+{
+    size_t byte_tot;
+    size_t byte_free;
+    cudaError_t cuda_status = cudaMemGetInfo(&byte_free, &byte_tot);
+
+    if (cuda_status != cudaSuccess)
+	    error("ERROR: cudaMemGetInfo failed. %s\n", cudaGetErrorString(cuda_status));
+
+
+    double dbyte_tot = (double)byte_tot;
+    double dbyte_free = (double)byte_free;
+    double dbyte_used = dbyte_tot - dbyte_free;
+
+    debug_printf(DP_INFO , "GPU memory usage: used = %.4f MiB, free = %.4f MiB, total = %.4f MiB\n", dbyte_used/MiBYTE, dbyte_free/MiBYTE, dbyte_tot/MiBYTE);
+}
+
 int cuda_devices(void)
 {
 	int count;
@@ -243,7 +263,7 @@ void* cuda_hostalloc(long N)
 {
 	void* ptr;
 	if (cudaSuccess != cudaHostAlloc(&ptr, N, cudaHostAllocDefault))
-		abort();
+	     error("abort");
 
 	insert(ptr, N, false);
 	return ptr;
@@ -349,6 +369,7 @@ const struct vec_ops gpu_ops = {
 	.zphsr = cuda_zphsr,
 	.zconj = cuda_zconj,
 	.zexpj = cuda_zexpj,
+	.zexp = cuda_zexp,
 	.zarg = cuda_zarg,
 	.zabs = cuda_zabs,
 
@@ -356,6 +377,10 @@ const struct vec_ops gpu_ops = {
 	.zdiv_reg = cuda_zdiv_reg,
 	.zfftmod = cuda_zfftmod,
 
+	.zmax = cuda_zmax,
+	.zle = cuda_zle,
+
+	.smax = cuda_smax,
 	.max = cuda_max,
 	.min = cuda_min,
 
