@@ -1,10 +1,10 @@
 /* Copyright 2013-2018. The Regents of the University of California.
- * Copyright 2015-2017. Martin Uecker.
+ * Copyright 2015-2018. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2012-2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2012-2018 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2014-2016 Frank Ong <frankong@berkeley.edu>
  * 2014-2018 Jon Tamir <jtamir@eecs.berkeley.edu>
  *
@@ -25,6 +25,7 @@
 #include "iter/misc.h"
 
 #include "linops/linop.h"
+#include "linops/fmac.h"
 #include "linops/sampling.h"
 #include "linops/someops.h"
 
@@ -42,21 +43,32 @@
 #include "misc/opts.h"
 
 #include "grecon/optreg.h"
+#include "grecon/italgo.h"
 
-
+#include "num/iovec.h"
+#include "num/ops.h"
 
 static const char usage_str[] = "<kspace> <sensitivities> <output>";
 static const char help_str[] = "Parallel-imaging compressed-sensing reconstruction.";
 
 
-static const struct linop_s* sense_nc_init(const long max_dims[DIMS], const long map_dims[DIMS], const complex float* maps, const long ksp_dims[DIMS], const long traj_dims[DIMS], const complex float* traj, struct nufft_conf_s conf, const complex float* weights, struct operator_s** precond_op, bool sms)
+
+static const struct linop_s* sense_nc_init(const long max_dims[DIMS], const long map_dims[DIMS], const complex float* maps, const long ksp_dims[DIMS], const long traj_dims[DIMS], const complex float* traj, struct nufft_conf_s conf, const long wgs_dims[DIMS], const complex float* weights, const long basis_dims[DIMS], const complex float* basis, struct operator_s** precond_op, bool sms)
 {
 	long coilim_dims[DIMS];
 	long img_dims[DIMS];
 	md_select_dims(DIMS, ~MAPS_FLAG, coilim_dims, max_dims);
 	md_select_dims(DIMS, ~COIL_FLAG, img_dims, max_dims);
 
-	const struct linop_s* fft_op = nufft_create(DIMS, ksp_dims, coilim_dims, traj_dims, traj, weights, conf);
+	long ksp_dims2[DIMS];
+	md_copy_dims(DIMS, ksp_dims2, ksp_dims);
+	ksp_dims2[COEFF_DIM] = max_dims[COEFF_DIM];
+	ksp_dims2[TE_DIM] = 1;
+
+	debug_print_dims(DP_INFO, DIMS, ksp_dims2);
+	debug_print_dims(DP_INFO, DIMS, coilim_dims);
+
+	const struct linop_s* fft_op = nufft_create2(DIMS, ksp_dims2, coilim_dims, traj_dims, traj, wgs_dims, weights, basis_dims, basis, conf);
 	const struct linop_s* maps_op = maps2_create(coilim_dims, map_dims, img_dims, maps);
 
 	if (sms) {
@@ -67,7 +79,9 @@ static const struct linop_s* sense_nc_init(const long max_dims[DIMS], const long
 		 */
 
 		const struct linop_s* fft_slice = linop_fft_create(DIMS, map_dims, SLICE_FLAG);
-		fft_op = linop_chain(fft_slice, fft_op);
+		const struct linop_s* tmp = fft_op;
+		fft_op = linop_chain(fft_slice, tmp);
+		linop_free(tmp);
 		linop_free(fft_slice);
 	}
 
@@ -91,7 +105,9 @@ int main_pics(int argc, char* argv[])
 
 	float bpsense_eps = -1.;
 
+	unsigned int shift_mode = 0;
 	bool randshift = true;
+	bool overlapping_blocks = false;
 	unsigned int maxiter = 30;
 	float step = -1.;
 
@@ -102,6 +118,7 @@ int main_pics(int argc, char* argv[])
 	// Read input options
 	struct nufft_conf_s nuconf = nufft_conf_defaults;
 	nuconf.toeplitz = true;
+	nuconf.lowmem = true;
 
 	float restrict_fov = -1.;
 	const char* pat_file = NULL;
@@ -121,11 +138,11 @@ int main_pics(int argc, char* argv[])
 	const char* image_start_file = NULL;
 	bool warm_start = false;
 
-	bool admm_dynamic_rho = false;
-	bool admm_dynamic_tau = false;
-	bool admm_relative_norm = false;
-	float admm_rho = iter_admm_defaults.rho;
-	unsigned int admm_maxitercg = iter_admm_defaults.maxitercg;
+	const char* basis_file = NULL;
+
+	struct admm_conf admm = { false, false, false, iter_admm_defaults.rho, iter_admm_defaults.maxitercg };
+
+	enum algo_t algo = ALGO_DEFAULT;
 
 	bool hogwild = false;
 	bool fast = false;
@@ -147,32 +164,34 @@ int main_pics(int argc, char* argv[])
 		OPT_UINT('i', &maxiter, "iter", "max. number of iterations"),
 		OPT_STRING('t', &traj_file, "file", "k-space trajectory"),
 		OPT_CLEAR('n', &randshift, "disable random wavelet cycle spinning"),
+		OPT_SET('N', &overlapping_blocks, "do fully overlapping LLR blocks"),
 		OPT_SET('g', &conf.gpu, "use GPU"),
 		OPT_UINT('G', &gpun, "gpun", "use GPU device gpun"),
 		OPT_STRING('p', &pat_file, "file", "pattern or weights"),
-		OPT_SELECT('I', enum algo_t, &ropts.algo, IST, "select IST"),
+		OPT_SELECT('I', enum algo_t, &algo, ALGO_IST, "select IST"),
 		OPT_UINT('b', &llr_blk, "blk", "Lowrank block size"),
 		OPT_SET('e', &eigen, "Scale stepsize based on max. eigenvalue"),
 		OPT_SET('H', &hogwild, "(hogwild)"),
-		OPT_SET('D', &admm_dynamic_rho, "(ADMM dynamic step size)"),
+		OPT_SET('D', &admm.dynamic_rho, "(ADMM dynamic step size)"),
 		OPT_SET('F', &fast, "(fast)"),
-		OPT_SET('J', &admm_relative_norm, "(ADMM residual balancing)"),
+		OPT_SET('J', &admm.relative_norm, "(ADMM residual balancing)"),
 		OPT_STRING('T', &image_truth_file, "file", "(truth file)"),
 		OPT_STRING('W', &image_start_file, "<img>", "Warm start with <img>"),
 		OPT_INT('d', &debug_level, "level", "Debug level"),
 		OPT_INT('O', &conf.rwiter, "rwiter", "(reweighting)"),
 		OPT_FLOAT('o', &conf.gamma, "gamma", "(reweighting)"),
-		OPT_FLOAT('u', &admm_rho, "rho", "ADMM rho"),
-		OPT_UINT('C', &admm_maxitercg, "iter", "ADMM max. CG iterations"),
+		OPT_FLOAT('u', &admm.rho, "rho", "ADMM rho"),
+		OPT_UINT('C', &admm.maxitercg, "iter", "ADMM max. CG iterations"),
 		OPT_FLOAT('q', &conf.cclambda, "cclambda", "(cclambda)"),
 		OPT_FLOAT('f', &restrict_fov, "rfov", "restrict FOV"),
-		OPT_SELECT('m', enum algo_t, &ropts.algo, ADMM, "select ADMM"),
+		OPT_SELECT('m', enum algo_t, &algo, ALGO_ADMM, "select ADMM"),
 		OPT_FLOAT('w', &scaling, "val", "inverse scaling of the data"),
 		OPT_SET('S', &scale_im, "re-scale the image after reconstruction"),
-		OPT_UINT('B', &loop_flags, "flags", "batch-mode"),
+		OPT_UINT('L', &loop_flags, "flags", "batch-mode"),
 		OPT_SET('K', &nuconf.pcycle, "randshift for NUFFT"),
+		OPT_STRING('B', &basis_file, "file", "temporal (or other) basis"),
 		OPT_FLOAT('P', &bpsense_eps, "eps", "Basis Pursuit formulation, || y- Ax ||_2 <= eps"),
-		OPT_SELECT('a', enum algo_t, &ropts.algo, PRIDU, "select Primal Dual"),
+		OPT_SELECT('a', enum algo_t, &algo, ALGO_PRIDU, "select Primal Dual"),
 		OPT_SET('M', &sms, "Simultaneous Multi-Slice reconstruction"),
 	};
 
@@ -187,8 +206,10 @@ int main_pics(int argc, char* argv[])
 	if (0 <= bpsense_eps)
 		conf.bpsense = true;
 
-	admm_dynamic_tau = admm_relative_norm;
+	admm.dynamic_tau = admm.relative_norm;
 
+	if (conf.bpsense)
+		nuconf.toeplitz = false;
 
 
 	long max_dims[DIMS];
@@ -200,7 +221,6 @@ int main_pics(int argc, char* argv[])
 	long traj_dims[DIMS];
 
 
-
 	// load kspace and maps and get dimensions
 
 	complex float* kspace = load_cfl(argv[1], DIMS, ksp_dims);
@@ -218,6 +238,17 @@ int main_pics(int argc, char* argv[])
 	map_flags |= FFT_FLAGS | SENS_FLAGS;
 
 
+
+	long basis_dims[DIMS];
+	complex float* basis = NULL;
+
+	if (NULL != basis_file) {
+
+		basis = load_cfl(basis_file, DIMS, basis_dims);
+		assert(!md_check_dimensions(DIMS, basis_dims, COEFF_FLAG | TE_FLAG));
+	}
+
+
 	complex float* traj = NULL;
 
 	if (NULL != traj_file)
@@ -227,6 +258,29 @@ int main_pics(int argc, char* argv[])
 	md_copy_dims(DIMS, max_dims, ksp_dims);
 	md_copy_dims(5, max_dims, map_dims);
 
+	assert(1 == ksp_dims[COEFF_DIM]);
+	long bmx_dims[DIMS];
+
+	if (NULL != basis_file) {
+
+		assert(basis_dims[TE_DIM] == ksp_dims[TE_DIM]);
+
+		max_dims[COEFF_DIM] = basis_dims[COEFF_DIM];
+
+		md_copy_dims(DIMS, bmx_dims, max_dims);
+		debug_printf(DP_INFO, "Basis: ");
+		debug_print_dims(DP_INFO, DIMS, bmx_dims);
+
+		max_dims[TE_DIM] = 1;
+
+		debug_printf(DP_INFO, "Max:   ");
+		debug_print_dims(DP_INFO, DIMS, max_dims);
+
+//		if (NULL != traj_file)
+//			nuconf.toeplitz = false;
+	}
+
+
 	md_select_dims(DIMS, ~COIL_FLAG, img_dims, max_dims);
 	md_select_dims(DIMS, ~MAPS_FLAG, coilim_dims, max_dims);
 
@@ -255,15 +309,28 @@ int main_pics(int argc, char* argv[])
 	if (hogwild)
 		debug_printf(DP_INFO, "Hogwild stepsize\n");
 
-	if (admm_dynamic_rho)
+	if (admm.dynamic_rho)
 		debug_printf(DP_INFO, "ADMM Dynamic stepsize\n");
 
-	if (admm_relative_norm)
+	if (admm.relative_norm)
 		debug_printf(DP_INFO, "ADMM residual balancing\n");
 
 	if (im_truth)
 		debug_printf(DP_INFO, "Compare to truth\n");
 
+	if (randshift)
+		shift_mode = 1;
+
+	if (overlapping_blocks) {
+
+		if (randshift)
+			debug_printf(DP_WARN, "Turning off random shifts\n");
+
+		shift_mode = 2;
+		debug_printf(DP_INFO, "Fully overlapping LLR blocks\n");
+	}
+
+
 
 	assert(!((conf.rwiter > 1) && (nuconf.toeplitz || conf.bpsense)));
 
@@ -338,10 +405,23 @@ int main_pics(int argc, char* argv[])
 	const struct linop_s* forward_op = NULL;
 	const struct operator_s* precond_op = NULL;
 
-	if (NULL == traj_file)
+	if (NULL == traj_file) {
+
 		forward_op = sense_init(max_dims, map_flags, maps);
-	else
-		forward_op = sense_nc_init(max_dims, map_dims, maps, ksp_dims, traj_dims, traj, nuconf, pattern, (struct operator_s**) &precond_op, sms);
+
+		// apply temporal basis
+
+		if (NULL != basis_file) {
+
+			const struct linop_s* basis_op = linop_fmac_create(DIMS, bmx_dims, COEFF_FLAG, TE_FLAG, ~(COEFF_FLAG | TE_FLAG), basis);
+			forward_op = linop_chain(forward_op, basis_op);
+		}
+
+	} else {
+
+		forward_op = sense_nc_init(max_dims, map_dims, maps, ksp_dims, traj_dims, traj, nuconf,
+				pat_dims, pattern, basis_dims, basis, (struct operator_s**)&precond_op, sms);
+	}
 
 
 	// apply scaling
@@ -376,7 +456,7 @@ int main_pics(int argc, char* argv[])
 		if (conf.bpsense) {
 
 			bpsense_eps /= scaling;
-			debug_printf(DP_DEBUG1, "scaling basis pursuit eps: %.3f\n", bpsense_eps);
+			debug_printf(DP_DEBUG1, "scaling basis pursuit eps: %.3e\n", bpsense_eps);
 		}
 	}
 
@@ -447,8 +527,9 @@ int main_pics(int argc, char* argv[])
 
 	// FIXME: re-initialize forward_op and precond_op
 
-	if ((NULL == traj_file) && !sms) {
+	if ((NULL == traj_file) && (0u != loop_flags) && !sms) { // FIXME: no basis
 
+		linop_free(forward_op);
 		forward_op = sense_init(max1_dims, map_flags, maps);
 
 		// basis pursuit requires the full forward model to add as a linop constraint
@@ -459,10 +540,20 @@ int main_pics(int argc, char* argv[])
 
 			linop_free(sample_op);
 			linop_free(forward_op);
+
 			forward_op = tmp;
 		}
 	}
 
+	double maxeigen = 1.;
+
+	if (eigen) {
+
+		maxeigen = estimate_maxeigenval(forward_op->normal);
+
+		debug_printf(DP_INFO, "Maximum eigenvalue: %.2e\n", maxeigen);
+
+	}
 
 
 	// initialize prox functions
@@ -470,40 +561,26 @@ int main_pics(int argc, char* argv[])
 	const struct operator_p_s* thresh_ops[NUM_REGS] = { NULL };
 	const struct linop_s* trafos[NUM_REGS] = { NULL };
 
-	opt_reg_configure(DIMS, img1_dims, &ropts, thresh_ops, trafos, llr_blk, randshift, conf.gpu);
+	opt_reg_configure(DIMS, img1_dims, &ropts, thresh_ops, trafos, llr_blk, shift_mode, conf.gpu);
 
 	if (conf.bpsense)
 		opt_bpursuit_configure(&ropts, thresh_ops, trafos, forward_op, kspace, bpsense_eps);
 
 	int nr_penalties = ropts.r;
 	struct reg_s* regs = ropts.regs;
-	enum algo_t algo = ropts.algo;
 
+	// choose algorithm
 
-	// initialize algorithm
-
-	italgo_fun2_t italgo = iter2_call_iter;
-	struct iter_call_s iter2_data;
-	SET_TYPEID(iter_call_s, &iter2_data);
-
-	iter_conf* iconf = CAST_UP(&iter2_data);
-
-	struct iter_conjgrad_conf cgconf = iter_conjgrad_defaults;
-	struct iter_fista_conf fsconf = iter_fista_defaults;
-	struct iter_ist_conf isconf = iter_ist_defaults;
-	struct iter_admm_conf mmconf = iter_admm_defaults;
-	struct iter_niht_conf ihconf = iter_niht_defaults;
-	struct iter_chambolle_pock_conf pdconf = iter_chambolle_pock_defaults;
-
-	if ((CG == algo) && (1 == nr_penalties) && (L2IMG != regs[0].xform))
-		algo = FISTA;
+	if (ALGO_DEFAULT == algo)
+		algo = italgo_choose(nr_penalties, regs);
 
 	if (conf.bpsense)
-		assert(ADMM == algo || PRIDU == algo);
-	else if (nr_penalties > 1)
-		algo = ADMM;
+		assert((ALGO_ADMM == algo) || (ALGO_PRIDU == algo));
 
-	if ((IST == algo) || (FISTA == algo)) {
+
+	// choose step size
+
+	if ((ALGO_IST == algo) || (ALGO_FISTA == algo) || (ALGO_PRIDU == algo)) {
 
 		// For non-Cartesian trajectories, the default
 		// will usually not work. TODO: The same is true
@@ -517,135 +594,28 @@ int main_pics(int argc, char* argv[])
 			step = 0.95;
 	}
 
-	if ((CG == algo) || (ADMM == algo))
+	if ((ALGO_CG == algo) || (ALGO_ADMM == algo))
 		if (-1. != step)
 			debug_printf(DP_INFO, "Stepsize ignored.\n");
 
-	if (eigen) {
+	step /= maxeigen;
 
-		double maxeigen = estimate_maxeigenval(forward_op->normal);
 
-		debug_printf(DP_INFO, "Maximum eigenvalue: %.2e\n", maxeigen);
+	// initialize algorithm
 
-		step /= maxeigen;
-	}
+	struct iter it = italgo_config(algo, nr_penalties, regs, maxiter, step, hogwild, fast, admm, scaling, warm_start);
 
-	switch (algo) {
+	if (ALGO_CG == algo)
+		nr_penalties = 0;
 
-		case CG:
-
-			debug_printf(DP_INFO, "conjugate gradients\n");
-
-			assert((0 == nr_penalties) || ((1 == nr_penalties) && (L2IMG == regs[0].xform)));
-
-			cgconf = iter_conjgrad_defaults;
-			cgconf.maxiter = maxiter;
-			cgconf.l2lambda = (0 == nr_penalties) ? 0. : regs[0].lambda;
-
-			iter2_data.fun = iter_conjgrad;
-			iter2_data._conf = CAST_UP(&cgconf);
-
-			nr_penalties = 0;
-
-			break;
-
-		case IST:
-
-			debug_printf(DP_INFO, "IST\n");
-
-			assert(1 == nr_penalties);
-
-			isconf = iter_ist_defaults;
-			isconf.maxiter = maxiter;
-			isconf.step = step;
-			isconf.hogwild = hogwild;
-
-			iter2_data.fun = iter_ist;
-			iter2_data._conf = CAST_UP(&isconf);
-
-			break;
-
-		case ADMM:
-
-			debug_printf(DP_INFO, "ADMM\n");
-
-			mmconf = iter_admm_defaults;
-			mmconf.maxiter = maxiter;
-			mmconf.maxitercg = admm_maxitercg;
-			mmconf.rho = admm_rho;
-			mmconf.hogwild = hogwild;
-			mmconf.fast = fast;
-			mmconf.dynamic_rho = admm_dynamic_rho;
-			mmconf.dynamic_tau = admm_dynamic_tau;
-			mmconf.relative_norm = admm_relative_norm;
-			mmconf.ABSTOL = 0.;
-			mmconf.RELTOL = 0.;
-
-			italgo = iter2_admm;
-			iconf = CAST_UP(&mmconf);
-
-			break;
-
-		case PRIDU:
-
-			debug_printf(DP_INFO, "Primal Dual\n");
-
-			assert(2 == nr_penalties);
-
-			pdconf = iter_chambolle_pock_defaults;
-
-			pdconf.maxiter = maxiter;
-			pdconf.sigma = 1. * scaling;
-			pdconf.tau = 1. / pdconf.sigma;
-			pdconf.theta = 1;
-			pdconf.decay = (hogwild ? .95 : 1);
-			pdconf.tol = 1E-4;
-
-			italgo = iter2_chambolle_pock;
-			iconf = CAST_UP(&pdconf);
-
-			break;
-
-		case FISTA:
-
-			debug_printf(DP_INFO, "FISTA\n");
-
-			assert(1 == nr_penalties);
-
-			fsconf = iter_fista_defaults;
-			fsconf.maxiter = maxiter;
-			fsconf.step = step;
-			fsconf.hogwild = hogwild;
-
-			iter2_data.fun = iter_fista;
-			iter2_data._conf = CAST_UP(&fsconf);
-
-			break;
-
-		case NIHT:
-
-			debug_printf(DP_INFO, "NIHT\n");
-
-			ihconf = iter_niht_defaults;
-			ihconf.maxiter = maxiter;
-			ihconf.do_warmstart=warm_start;
-
-			italgo = iter2_niht;
-			iconf = CAST_UP(&ihconf);
-
-			conf.gpu = false; // gpu not implemented, disable
-
-			break;		
-
-		default:			
-			assert(0);
-	}
-
-	bool trafos_cond = ((PRIDU == algo) || (ADMM == algo) || ((NIHT == algo) && (regs[0].xform == NIHTWAV)));
+	bool trafos_cond = (   (ALGO_PRIDU == algo)
+			    || (ALGO_ADMM == algo)
+			    || (   (ALGO_NIHT == algo)
+				&& (regs[0].xform == NIHTWAV)));
 	
 	const struct operator_s* op = sense_recon_create(&conf, max1_dims, forward_op,
-				pat1_dims, (NULL != traj_file) ? NULL : (conf.bpsense ? NULL : pattern1),
-				italgo, iconf, image_start1, nr_penalties, thresh_ops,
+				pat1_dims, ((NULL != traj_file) || conf.bpsense) ? NULL : pattern1,
+				it.italgo, it.iconf, image_start1, nr_penalties, thresh_ops,
 				trafos_cond ? trafos : NULL, precond_op);
 
 	long strsx[2][DIMS];
@@ -666,7 +636,8 @@ int main_pics(int argc, char* argv[])
 	if (0 != loop_flags) {
 
 		op = operator_copy_wrapper(2, strs, op);
-		op = operator_loop(DIMS, loop_dims, op);
+		// op = operator_loop(DIMS, loop_dims, op);
+		op = operator_loop_parallel(DIMS, loop_dims, op, loop_flags, conf.gpu);
 	}
 
 	operator_apply(op, DIMS, img_dims, image, DIMS, conf.bpsense ? img_dims : ksp_dims, conf.bpsense ? NULL : kspace);
@@ -675,7 +646,7 @@ int main_pics(int argc, char* argv[])
 
 	opt_reg_free(&ropts, thresh_ops, trafos);
 
-
+	italgo_config_free(it);
 
 	if (scale_im)
 		md_zsmul(DIMS, img_dims, image, image, scaling);
@@ -704,10 +675,14 @@ int main_pics(int argc, char* argv[])
 	if (image_start)
 		unmap_cfl(DIMS, img_dims, image_start);
 
+	xfree(pat_file);
+	xfree(traj_file);
+
 	double end_time = timestamp();
 
 	debug_printf(DP_INFO, "Total Time: %f\n", end_time - start_time);
-	exit(0);
+
+	return 0;
 }
 
 
