@@ -83,7 +83,7 @@ int main_rof(int argc, char* argv[])
 	
 	unmap_cfl(DIMS, dims, in_data);
 	unmap_cfl(DIMS, dims, out_data);
-	exit(0);
+	return 0;
 }
 
 
