@@ -66,7 +66,7 @@ int main_svd(int argc, char* argv[])
 	unmap_cfl(N, dimsU, U);
 	unmap_cfl(N, dimsS, S);
 	unmap_cfl(N, dimsVH, VH);
-	exit(0);
+	return 0;
 }
 
 
