@@ -1,10 +1,10 @@
-/* Copyright 2013-2015 The Regents of the University of California.
+/* Copyright 2013-2018 The Regents of the University of California.
  * All rights reserved. Use of this source code is governed by 
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors: 
  * 2013, 2015 Martin Uecker <uecker@eecs.berkeley.edu>
- * 2015 Jonathan Tamir <jtamir@eecs.berkeley.edu>
+ * 2015, 2018 Jon Tamir <jtamir@eecs.berkeley.edu>
  */
 
 #include <stdlib.h>
@@ -14,15 +14,18 @@
 #include <strings.h>
 #include <complex.h>
 #include <stdbool.h>
+#include <math.h>
 
 #include "num/multind.h"
 #include "num/init.h"
+#include "num/flpmath.h"
 
 #include "misc/misc.h"
 #include "misc/debug.h"
 #include "misc/mmio.h"
 #include "misc/png.h"
 #include "misc/dicom.h"
+#include "misc/opts.h"
 
 #ifndef DIMS
 #define DIMS 16
@@ -39,8 +42,18 @@ static const char help_str[] = "Create magnitude images as png or proto-dicom.\n
 				"be used for the image, and the other dimensions\n"
 				"will be looped over.\n";
 
+// from view:src/draw.c
+static double clamp(double a, double b, double x)
+{
+	return (x < a) ? a : ((x > b) ? b : x);
+}
 
-static void toimg(bool dicom, const char* name, long inum, float max, long h, long w, const complex float* data)
+static double windowing(double g, double a, double b, double x)
+{
+	return pow(clamp(0., 1., (x - a) / (b - a)), g);
+}
+
+static void toimg(bool dicom, bool use_windowing, const char* name, long inum, float gamma, float contrast, float window, float scale, long h, long w, const complex float* data)
 {
 	int len = strlen(name);
 	assert(len >= 1);
@@ -54,7 +67,8 @@ static void toimg(bool dicom, const char* name, long inum, float max, long h, lo
 
 		for (int j = 0; j < w; j++) {
 
-			unsigned int value = max_val * (cabsf(data[j * h + i]) / max);
+			double val = cabsf(data[j * h + i]) / scale;
+			unsigned int value = (unsigned int)(max_val * (use_windowing ? windowing(gamma, contrast, window, val) : val));
 
 			if (!dicom) {
 
@@ -75,7 +89,7 @@ static void toimg(bool dicom, const char* name, long inum, float max, long h, lo
 }
 
 
-static void toimg_stack(const char* name, bool dicom, const long dims[DIMS], const complex float* data)
+static void toimg_stack(const char* name, bool dicom, bool single_scale, bool use_windowing, float gamma, float contrast, float window, const long dims[DIMS], const complex float* data)
 {
 	long data_size = md_calc_size(DIMS, dims); 
 
@@ -91,9 +105,6 @@ static void toimg_stack(const char* name, bool dicom, const long dims[DIMS], con
 	for (long i = 0; i < data_size; i++)
 		max = MAX(cabsf(data[i]), max);
 
-	if (0. == max)
-		max = 1.;
-
 	int len = strlen(name);
 	assert(len >= 1);
 
@@ -112,7 +123,20 @@ static void toimg_stack(const char* name, bool dicom, const long dims[DIMS], con
 		else
 			sprintf(name_i, "%s.%s", name, dicom ? "dcm" : "png");
 
-		toimg(dicom, name_i, i, max, sq_dims[0], sq_dims[1], data + i * img_size);
+		float scale = 0.;
+
+		if (use_windowing)
+			scale = md_znorm(2, sq_dims, data + i * img_size) / md_calc_size(2, sq_dims);
+		else if (single_scale)
+			scale = max;
+		else 
+			for (long j = 0; j < md_calc_size(2, sq_dims); j++)
+				scale = MAX(cabsf(data[i * img_size + j]), scale);
+
+		if (0. == scale)
+			scale = 1.;
+
+		toimg(dicom, use_windowing, name_i, i, gamma, contrast, window, scale, sq_dims[0], sq_dims[1], data + i * img_size);
 	}
 
 	debug_printf(DP_INFO, "done.\n", num_imgs);
@@ -121,12 +145,27 @@ static void toimg_stack(const char* name, bool dicom, const long dims[DIMS], con
 
 int main_toimg(int argc, char* argv[])
 {
-	bool dicom = mini_cmdline_bool(&argc, argv, 'd', 2, usage_str, help_str);
+	float gamma = 1.;
+	float contrast = 0.;
+	float window = 750.;
+	bool use_windowing = false;
+	bool single_scale = true;
+	bool dicom = false;
+
+	const struct opt_s opts[] = {
+
+		OPT_FLOAT('g', &gamma, "gamma", "gamma level"),
+		OPT_FLOAT('c', &contrast, "contrast", "contrast level"),
+		OPT_FLOAT('w', &window, "window", "window level"),
+		OPT_SET('d', &dicom, "write to dicom format (deprecated, use extension .dcm)"),
+		OPT_CLEAR('m', &single_scale, "re-scale each image"),
+		OPT_SET('W', &use_windowing, "use dynamic windowing"),
+	};
+
+	cmdline(&argc, argv, 2, 2, usage_str, help_str, ARRAY_SIZE(opts), opts);
 
 	num_init();
 
-	// -d option is deprecated
-
 	char* ext = rindex(argv[2], '.');
 
 	if (NULL != ext) {
@@ -145,11 +184,11 @@ int main_toimg(int argc, char* argv[])
 	long dims[DIMS];
 	complex float* data = load_cfl(argv[1], DIMS, dims);
 
-	toimg_stack(argv[2], dicom, dims, data);
+	toimg_stack(argv[2], dicom, single_scale, use_windowing, gamma, contrast, window, dims, data);
 
 	unmap_cfl(DIMS, dims, data);
 
-	exit(0);
+	return 0;
 }
 
 
